/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/*
 * Pan - A Newsreader for Gtk+
 * Copyright (C) 2002-2006  Charles Kerr <charles@rebelbase.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _UI_Prefs_h_
#define _UI_Prefs_h_

#include <map>
#include <string>
#include <pan/general/string-view.h>
#include <gtk/gtk.h>

namespace pan
{
  struct SaveWidgetAllocationsToPrefs
  {
    virtual ~SaveWidgetAllocationsToPrefs () {}
    virtual bool save_widget_allocations_to_prefs () const = 0;
  };
    
  /**
   * UI-oriented prefs.
   * @ingroup GUI
   */
  class UI_Prefs
  {
    private:
      std::string _filename;

    private:
      void save ();
      void serialize (std::ostream&) const;
      void set_from_file (const StringView& filename);
      void set_from_xml (const StringView& xml_string);

    public:

      bool get_flag (const StringView& key, bool fallback_value) const;

      void set_flag (const StringView& key, bool);

      void set_window (const StringView& key, GtkWindow* window,
                       int default_x, int default_y,
                       int default_width, int default_height);

      void set_widget (const StringView& key, GtkWidget* widget,
                       SaveWidgetAllocationsToPrefs* sentinel,
                       int default_width=-1, int default_height=-1);

      void set_default_geometry (const StringView&, int, int, int, int);
      void set_geometry (const StringView&, int, int, int, int);
      bool get_geometry (const StringView&, int&, int&, int&, int&) const;
      bool do_save_widget_allocations_to_prefs (const StringView&) const;

    public:
      UI_Prefs (const StringView& filename);
      ~UI_Prefs ();

    private:
      struct Geometry {
        int x, y, w, h;
        Geometry(int xx, int yy, int ww, int hh): x(xx), y(yy), w(ww), h(hh) {}
        Geometry(): x(0), y(0), w(0), h(0) {}
      };
      typedef std::map<std::string,Geometry> window_to_geometry_t;
      window_to_geometry_t _window_to_geometry;
      typedef std::map<std::string,SaveWidgetAllocationsToPrefs*> sentinels_t;
      sentinels_t _sentinels;
      static void size_allocated_cb (GtkWidget*, GtkAllocation*, gpointer);

    private:
      typedef std::map<std::string,bool> flags_t;
      mutable flags_t _flags;
  };
}

#endif
